var Experience4DPlayer = function () {
	return {
		iframeContainer: null,
		playingScope: null,
		playingScenes: [],
		playingAnimations: [],
		currentPlayingSceneIndex: -1,
		currentPlayingAnimationIndex: -1,
		totalDuration: 0,
		playButton: null,
		progressBar: null,
		editor: null,
		sceneHtmlElements: [],
		currentPlayingSceneNumberElement: null,
		currentPlayingSceneTitleElement: null,
		startPlayingMilliseconds: null,
		isPlaying: false,
		nextSceneButton: null,
		previousSceneButton: null,

		playExperience: function () {
			this.isPlaying = true;
			this.clearPlayerTimeoutsAndIntervals();
			this.configureAnimations();
			this.configureScenes();
			this.calculateTotalScenesDuration();

			if (this.playingAnimations.length === 0) {
				if (this.playingScenes.length === 0) {
					return;
				}
			}

			if (this.playExperienceCallback) {
				this.playExperienceCallback();
			}

			if (this.modelViewer && this.modelViewer.loaded) {
				this.playLoadedModelViewer();
				this.showPauseButton();
				return;
			}

			var attempts = 10;
			var scope = this;
			scope.previewIframeIntervalID = setInterval(function () {
				attempts -= 1;
				if (attempts === 0) {
					clearInterval(scope.previewIframeIntervalID);
					return;
				}

				if (!scope.iframeContainer) {
					return;
				}

				var modelViewer = scope.iframeContainer.querySelector("model-viewer");
				if (!modelViewer) {
					return;
				}
				scope.modelViewer = modelViewer;

				if (modelViewer.loaded) {
					scope.playLoadedModelViewer();
				} else {
					modelViewer.addEventListener("load", function () {
						scope.playLoadedModelViewer();
					});
				}
				clearInterval(scope.previewIframeIntervalID);
			}, 700);

			scope.showPauseButton();
		},
		playLoadedModelViewer: function () {
			this.configureLoadedModelViewer();
			if (this.shouldResumePlaying) {
				this.shouldResumePlaying = false;
				// resume playing from the position of the input in the seek bar
				var seekBarValue = this.seekBar.value;
				var resumeInformation = this.getResumeInformationFromSeekBarValue(seekBarValue);
				this.resumeExperience(resumeInformation.resumeSceneInformation, resumeInformation.resumeAnimationInformation);
			} else if (this.shouldResumePlayingFromBeginningOfScene) {
				this.shouldResumePlayingFromBeginningOfScene = false;
				var playSceneInformation = this.calculateInformationToPlayScene(this.currentPlayingSceneIndex);
				this.playedSeconds = playSceneInformation.playedSeconds;
				var playAnimationInformation = this.calculateResumePlayAnimationInformation(this.playedSeconds);
				this.resumeExperience(playSceneInformation, playAnimationInformation);
			} else {
				this.playExperienceFromTheBeginning();
			}
		},
		configureLoadedModelViewer: function () {
			this.modelViewer.interpolationDecay = this.interpolationDecay;
		},
		getResumeInformationFromSeekBarValue: function (seekBarValue) {
			// resume from 0 if the player ended
			var inputPercentage = seekBarValue >= 100 ? 0 : seekBarValue;
			var seconds = this.getSecondsFromPercentage(inputPercentage);

			// if in preview mode resume from the selected position
			if (this.isPreview && seekBarValue >= 100) {
				seconds = this.totalDuration;
			} else if (this.isPreview && seekBarValue <= 0) {
				seconds = 0;
			}

			var resumeSceneInformation = this.calculateResumePlaySceneInformation(seconds);
			var resumeAnimationInformation = this.calculateResumePlayAnimationInformation(seconds);

			return {
				resumeSceneInformation: resumeSceneInformation,
				resumeAnimationInformation: resumeAnimationInformation
			};
		},
		getSecondsFromPercentage: function (percentage) {
			return percentage / 100 * this.totalDuration;
		},
		calculateResumePlaySceneInformation: function (seconds) {
			// iterate all scenes
			// substract the duration of each scene from the seconds
			// if the result is negative, that scene is the current scene playing
			// the remaining seconds is the already played time
			// the negative result, converted to a positive value is the remaining duration
			var remainingSeconds = seconds;
			var playedSeconds = 0;
			var remainingDuration = 0;
			var currentSceneIndex = -1;

			for (var i = 0; i < this.playingScenes.length; i++) {
				var sceneDuration = this.playingScenes[i].duration;
				currentSceneIndex = i;

				if (remainingSeconds - sceneDuration < 0) {
					// current scene found
					playedSeconds += remainingSeconds;
					remainingDuration = Math.abs(remainingSeconds - sceneDuration);
					break;
				} else {
					// move to next scene
					playedSeconds += sceneDuration;
					remainingSeconds -= sceneDuration;
				}
			}

			return {
				currentSceneIndex: currentSceneIndex,
				playedSeconds: playedSeconds,
				remainingDuration: remainingDuration
			};
		},
		calculateResumePlayAnimationInformation: function (seconds) {
			var playingAnimations = this.playingAnimations;
			var defaultInvalidAnimation = {
				currentAnimationIndex: -1,
				playedSeconds: 0,
				remainingDuration: 0
			};

			if (!playingAnimations || playingAnimations.length === 0) {
				return defaultInvalidAnimation;
			}

			var currentAnimationIndex = -1;
			var totalPlayedTime = 0;
			var animationDuration;
			var lastAnimation = this.playingAnimations[this.playingAnimations.length - 1];
			var lastAnimationStopTime = lastAnimation.stopTime;

			if (this.isPreview && seconds >= lastAnimationStopTime) {
				return {
					currentAnimationIndex: this.playingAnimations.length - 1,
					playedSeconds: lastAnimation.stopTime - lastAnimation.startTime,
					remainingDuration: 0
				};
			}

			if (seconds > lastAnimationStopTime) {
				// Player has finished playing all animations
				return defaultInvalidAnimation;
			}

			for (var i = 0; i < this.playingAnimations.length; i++) {
				var animation = this.playingAnimations[i];
				animationDuration = animation.stopTime - animation.startTime;

				if (seconds >= animation.startTime
					&& seconds < animation.stopTime
					&& animation.startTime < animation.stopTime) {
					// current animation
					currentAnimationIndex = i;
					totalPlayedTime = seconds - animation.startTime;
					break;
				} else if (seconds >= animation.stopTime) {
					// animation already played
					totalPlayedTime += animationDuration;
				} else {
					// animation not played yet
					break;
				}
			}

			var remainingDuration = animationDuration - totalPlayedTime;

			return {
				currentAnimationIndex: currentAnimationIndex,
				playedSeconds: totalPlayedTime,
				remainingDuration: remainingDuration > 0 ? remainingDuration : 0
			};
		},
		calculateInformationToPlayScene: function (sceneIndex) {
			var defaultInvalidScene = {
				currentAnimationIndex: -1,
				playedSeconds: 0,
				remainingDuration: 0
			};

			if (!this.playingScenes || this.playingScenes.length === 0) {
				return defaultInvalidScene;
			}

			var numScenes = this.playingScenes.length;

			// handle negative index by starting from the end
			if (sceneIndex < 0) {
				sceneIndex = numScenes + sceneIndex;
			}

			// make the index circular
			sceneIndex = sceneIndex % numScenes;

			var playedSeconds = 0;
			for (var i = 0; i < sceneIndex; i++) {
				playedSeconds += this.playingScenes[i].duration;
			}

			var remainingDuration = this.playingScenes[sceneIndex].duration;

			return {
				currentSceneIndex: sceneIndex,
				playedSeconds: playedSeconds,
				remainingDuration: remainingDuration
			};
		},
		resumeExperience: function (resumeSceneInformation, resumeAnimationInformation) {
			this.currentPlayingSceneIndex = resumeSceneInformation.currentSceneIndex;
			this.startPlayingMilliseconds = this.getDateBeforeSeconds(resumeSceneInformation.playedSeconds);
			this.playCurrentScene(resumeSceneInformation);

			this.currentPlayingAnimationIndex = resumeAnimationInformation.currentAnimationIndex;
			this.playCurrentAnimation(resumeAnimationInformation);
		},
		getDateBeforeSeconds: function (seconds) {
			return (new Date(Date.now() - (seconds * 1000))).getTime();
		},
		playExperienceFromTheBeginning: function () {
			//reset values
			this.currentPlayingSceneIndex = -1;
			this.currentPlayingAnimationIndex = -1;
			this.startPlayingMilliseconds = null;
			this.playNextAnimation();
			this.playNextScene();
		},
		pauseExperience: function (shouldResumePlaying, shouldResumePlayingFromNextScene, forceUpdateProgress) {
			this.clearPlayerTimeoutsAndIntervals();
			this.isPlaying = false;
			this.pausedAtTime = Date.now();
			this.shouldResumePlaying = shouldResumePlaying;
			this.shouldResumePlayingFromNextScene = shouldResumePlayingFromNextScene;
			this.showPlayButton();
			this.modelViewer.pause();

			if (forceUpdateProgress) {
				// force the execution because the intervalID is already cleaned
				this.showProgressInSeekBarCallback();
			}
		},
		showPauseButton: function () {
			if (!this.playButton.classList.contains("experience4d-pause")) {
				this.playButton.classList.add('experience4d-pause');
			}
		},
		showPlayButton: function () {
			if (this.playButton.classList.contains("experience4d-pause")) {
				this.playButton.classList.remove("experience4d-pause");
			}
		},
		clearPlayerTimeoutsAndIntervals: function () {
			var intervals = ["scenePlayerIntervalID", "previewIframeIntervalID"];
			for (var j = 0; j < intervals.length; j++) {
				var intervalID = this[intervals[j]];
				if (intervalID) {
					clearInterval(intervalID);
					this[intervals[j]] = null;
				}
			}

			var timeouts = ["startAnimationTimeoutID", "endAnimationTimeoutID", "endSceneTimeoutID", "endSceneTimeoutID"];
			for (var i = 0; i < timeouts.length; i++) {
				var timeoutID = this[timeouts[i]];
				if (timeoutID) {
					clearTimeout(timeoutID);
					this[timeouts[i]] = null;
				}
			}
		},
		playNextAnimation: function () {
			this.currentPlayingAnimationIndex += 1;
			this.playCurrentAnimation();
		},
		playCurrentAnimation: function (resumeAnimationInformation) {
			if (this.playingAnimations.length === 0
				|| this.currentPlayingAnimationIndex >= this.playingAnimations.length
				|| this.currentPlayingAnimationIndex < 0) {
				if (this.startAnimationTimeoutID) {
					clearTimeout(this.startAnimationTimeoutID);
				}
				return;
			}

			var currentPlayingAnimation = this.playingAnimations[this.currentPlayingAnimationIndex];
			var startDelay = 0;

			if (resumeAnimationInformation) {
				startDelay = 0;
			} else if (this.currentPlayingAnimationIndex === 0) {
				startDelay = currentPlayingAnimation.startTime;
			} else if (this.currentPlayingAnimationIndex > 0) {
				var previousAnimation = this.playingAnimations[this.currentPlayingAnimationIndex - 1];
				if (previousAnimation.stopTime) {
					startDelay = currentPlayingAnimation.startTime - previousAnimation.stopTime;
				}
			}

			if (this.startAnimationTimeoutID) {
				clearTimeout(this.startAnimationTimeoutID);
			}

			var currentPlayingAnimationDuration = resumeAnimationInformation
				? resumeAnimationInformation.remainingDuration
				: currentPlayingAnimation.stopTime - currentPlayingAnimation.startTime;

			// delay the start of the animation, for when the start time is not 0
			var scope = this;
			scope.startAnimationTimeoutID = setTimeout(function () {
				scope.modelViewer.animationName = currentPlayingAnimation.name;
				scope.modelViewer.currentTime = 0;
				scope.modelViewer.play();

				if (currentPlayingAnimation.stopTime
					&& currentPlayingAnimation.stopTime > currentPlayingAnimation.startTime) {
					scope.endAnimationTimeoutID = setTimeout(function () {
						scope.modelViewer.pause();
						scope.playNextAnimation();
					}, (currentPlayingAnimationDuration) * 1000);
				}
			}, startDelay * 1000);
		},
		playNextScene: function () {
			this.currentPlayingSceneIndex += 1;
			this.playCurrentScene();
		},
		playCurrentScene: function (resumeSceneInformation) {
			// if there is no duration defined, do not play
			if (this.totalDuration === 0) {
				return;
			}

			if (this.playingScenes.length === 0
				|| this.currentPlayingSceneIndex >= this.playingScenes.length
				|| this.currentPlayingSceneIndex < 0) {
				if (this.endSceneTimeoutID) {
					clearTimeout(this.endSceneTimeoutID);
				}
				return;
			}

			// save the start time of the first scene, to calculate the progress after
			if (this.currentPlayingSceneIndex === 0 && !resumeSceneInformation) {
				this.startPlayingMilliseconds = Date.now();
			}

			var currentScene = this.playingScenes[this.currentPlayingSceneIndex];
			// if a scene does not have a duration defined, try playing the next one
			if (currentScene.duration === 0) {
				if (this.endSceneTimeoutID) {
					clearTimeout(this.endSceneTimeoutID);
				}
				this.playNextScene();
				return;
			}

			// process current scene
			this.modelViewer.cameraTarget = currentScene.cameraTarget;
			this.modelViewer.cameraOrbit = currentScene.cameraOrbit;
			this.modelViewer.fieldOfView = currentScene.fieldOfView;
			this.showSceneContentInPanel(this.currentPlayingSceneIndex);
			this.displaySceneInPlayer(this.currentPlayingSceneIndex);

			// clear the previous scene timeout, to be safe
			if (this.endSceneTimeoutID) {
				clearTimeout(this.endSceneTimeoutID);
			}

			var scenePlayDuration = resumeSceneInformation ? resumeSceneInformation.remainingDuration : currentScene.duration;

			var scope = this;
			// play the current scene for the given duration
			scope.endSceneTimeoutID = setTimeout(function () {
				if (currentScene.stopAtEnd) {
					scope.modelViewer.pause();
				}

				if (scope.forcePauseAtEnd) {
					scope.forcePauseAtEnd = false;
					scope.pauseExperience(true, false, true);
				}

				scope.playNextScene();
			}, (scenePlayDuration) * 1000);

			scope.showProgressInSeekBar();
		},
		showProgressInSeekBar: function () {
			if (this.scenePlayerIntervalID) {
				return;
			}

			// interval used to update the seek bar
			var scope = this;
			scope.scenePlayerIntervalID = setInterval(function () {
				scope.showProgressInSeekBarCallback();
			}, 20);
		},
		showProgressInSeekBarCallback: function() {
			var nowMilliseconds = Date.now();
			if (!this.isPlaying) {
				clearInterval(this.scenePlayerIntervalID);
				if (this.pausedAtTime) {
					nowMilliseconds = this.pausedAtTime;
					this.pausedAtTime = null;
				}
			}

			var elapsedSeconds = (nowMilliseconds - this.startPlayingMilliseconds) / 1000;
			var progress = (elapsedSeconds / this.totalDuration) * 100;

			// player already ended
			if (progress >= 100) {
				this.updateSeekBarProgress(100);
				this.showPlayedTimeInSeekBar(this.totalDuration);
				this.pauseExperience(true, false, false);
				return;
			}

			// update player
			this.updateSeekBarProgress(progress);
			this.showPlayedTimeInSeekBar(elapsedSeconds);
		},
		showPlayedTimeInSeekBar: function (seconds) {
			var h = Math.floor(seconds / 3600);
			var m = Math.floor(seconds % 3600 / 60);
			var s = Math.floor(seconds % 60);
			var hDisplay = h > 0 ? (h < 10 ? '0' + h + ':' : h + ':') : '';
			var mDisplay = m > 0 || h > 0 ? (m < 10 ? "0" + m : m) + ":" : "00:";
			var sDisplay = s < 10 ? '0' + s : s;
			this.currentPlayedTimeElement.textContent = hDisplay + mDisplay + sDisplay;
		},
		showSceneContentInPanel: function (currentPlayingSceneIndex) {
			if (!this.playingScenes) {
				return;
			}

			if (currentPlayingSceneIndex >= this.playingScenes.length || currentPlayingSceneIndex < 0) {
				return;
			}

			var sceneContentHtml;
			if (this.editor) {
				this.editor.selectedScene = this.editor.sceneElements[currentPlayingSceneIndex];
				if (this.editor.selectedScene) {
					var titleElem = this.editor.selectedScene.querySelector('[data-type="Title"]');
					this.editor.activePlugin.displaySceneOptionsInToolbar(this.editor, titleElem);

					sceneContentHtml = this.editor.activePlugin.GUI.getBodyHTMLElement("smc-item-preview-scene-content-right");
					// add 1 because position of the element is one-based in the server
					this.editor.activePlugin.displaySceneContentInPreview(sceneContentHtml, currentPlayingSceneIndex + 1);
				}
			} else if (this.sceneHtmlElements && this.sceneHtmlElements.length > 0) {
				Experience4DUtil.displaySceneContentInHTML5(this, this.modelViewer, currentPlayingSceneIndex);
			} else if (this.playingScope.GUI) {
				sceneContentHtml = this.playingScope.GUI.getBodyHTMLElement("smc.doc.viewer.scene.content.right");
				if (this.playingScope.displaySceneContentInPreview) {
					// add 1 because position of the element is one-based in the server
					this.playingScope.displaySceneContentInPreview(sceneContentHtml, currentPlayingSceneIndex + 1);
				}
			}
		},
		displaySceneInPlayer: function (currentPlayingSceneIndex) {
			if (!this.playingScenes) {
				return;
			}

			var playingScenesLength = this.playingScenes.length;
			if (currentPlayingSceneIndex >= playingScenesLength || currentPlayingSceneIndex < 0) {
				return;
			}

			var currentScene = this.playingScenes[currentPlayingSceneIndex];
			this.currentPlayingSceneNumberElement.textContent = String(currentPlayingSceneIndex + 1) + "/" + playingScenesLength;
			this.currentPlayingSceneTitleElement.textContent = currentScene.title;
		},
		setSceneHtmlElements: function (sceneHtmlElements) {
			this.sceneHtmlElements = sceneHtmlElements;
		},
		calculateTotalScenesDuration: function () {
			if (!this.playingScenes) {
				return 0;
			}

			this.totalDuration = this.playingScenes.reduce(function (total, scene) {
				return total + scene.duration;
			}, 0);
		},
		configurePlayButton: function (button) {
			if (!button) {
				return;
			}

			var scope = this;
			scope.playButton = button;

			scope.playButton.addEventListener("click", function () {
				if (!scope.playButton.classList.contains('experience4d-pause')) {
					scope.playExperience();
				} else {
					scope.pauseExperience(true);
				}
			});
		},
		configureSeekBar: function (seekBar) {
			if (!seekBar) {
				return;
			}

			var scope = this;
			scope.seekBar = seekBar;

			scope.seekBar.addEventListener("input", function (e) {
				if (scope.isPlaying) {
					scope.pauseExperience(true);
					scope.updateSeekBarProgress(e.target.value);
					scope.playExperience();
				} else {
					scope.previewExperienceAtPercentage(e.target.value);
				}
			});
		},
		previewExperienceAtPercentage: function (percentage) {
			this.isPreview = true;
			this.clearPlayerTimeoutsAndIntervals();
			this.configureAnimations();
			this.configureScenes();
			this.calculateTotalScenesDuration();
			this.updateSeekBarProgress(percentage);

			var previewInformation = this.getResumeInformationFromSeekBarValue(percentage);

			var sceneInformation = previewInformation.resumeSceneInformation;
			var animationInformation = previewInformation.resumeAnimationInformation;

			var currentPlayingSceneIndex = sceneInformation.currentSceneIndex;
			var currentPlayingAnimationIndex = animationInformation.currentAnimationIndex;

			var currentScene = this.playingScenes[currentPlayingSceneIndex];
			if (!currentScene) {
				this.isPreview = false;
				return;
			}

			this.modelViewer.cameraOrbit = currentScene.cameraOrbit;
			this.modelViewer.fieldOfView = currentScene.fieldOfView;

			var currentAnimation = this.playingAnimations[currentPlayingAnimationIndex];
			if (currentAnimation) {
				this.modelViewer.animationName = currentAnimation.name;
				this.modelViewer.play();
				this.modelViewer.pause();
				this.modelViewer.currentTime = animationInformation.playedSeconds % this.modelViewer.duration;
			}

			var nowMilliseconds = Date.now();
			var startPlayingMilliseconds = this.getDateBeforeSeconds(sceneInformation.playedSeconds);
			var elapsedSeconds = (nowMilliseconds - startPlayingMilliseconds) / 1000;
			this.showPlayedTimeInSeekBar(elapsedSeconds);
			this.displaySceneInPlayer(currentPlayingSceneIndex);
			this.showSceneContentInPanel(currentPlayingSceneIndex);

			this.shouldResumePlaying = true;
			this.isPreview = false;
		},

		previewExperienceScene: function (currentIndex) {
			var currentScene = this.playingScenes[currentIndex];
			if (!currentScene) {
				this.isPreview = false;
				return;
			}
			this.playSelectedScene(currentIndex, true);
			this.pauseExperience(true, false, true);
		},
		updateSeekBarProgress: function (value) {
			var min = 0;
			var max = 100;

			this.seekBar.value = value;
			this.seekBar.style.backgroundSize = (value - min) * 100 / (max - min) + '% 100%';
		},
		configureNextSceneButton: function (button) {
			var scope = this;
			scope.nextSceneButton = button;
			scope.nextSceneButton.addEventListener("click", function () {
				scope.shouldResumePlayingFromBeginningOfScene = true;
				if (scope.isPlaying) {
					scope.pauseExperience(false, true);
					scope.incrementSceneOnNextSceneButtonClick();
					scope.playExperience();
				} else {
					scope.incrementSceneOnNextSceneButtonClick();
					scope.calculateTotalScenesDuration();
					var playSceneInformation = scope.calculateInformationToPlayScene(scope.currentPlayingSceneIndex);
					var playedPercentage = scope.getPercentageFromSeconds(playSceneInformation.playedSeconds);
					scope.previewExperienceAtPercentage(playedPercentage);
				}
			});
		},
		playSelectedScene: function(sceneIndex, forcePauseAtEnd) {
			if (this.isPlaying) {
				this.pauseExperience();
			}

			this.shouldResumePlaying = false;
			this.shouldResumePlayingFromBeginningOfScene = true;
			this.forcePauseAtEnd = forcePauseAtEnd;
			this.currentPlayingSceneIndex = sceneIndex;
			this.playExperience();
		},
		getPercentageFromSeconds: function (seconds) {
			return (seconds / this.totalDuration) * 100;
		},
		incrementSceneOnNextSceneButtonClick: function () {
			if (this.currentPlayingSceneIndex === -1) {
				this.currentPlayingSceneIndex = 1;
			} else {
				this.currentPlayingSceneIndex += 1;
				this.currentPlayingSceneIndex = this.currentPlayingSceneIndex % this.playingScenes.length;
			}
		},
		configurePreviousSceneButton: function (button) {
			var scope = this;
			scope.previousSceneButton = button;
			scope.previousSceneButton.addEventListener("click", function () {
				scope.shouldResumePlayingFromBeginningOfScene = true;
				if (scope.isPlaying) {
					scope.pauseExperience(false, true);
					scope.currentPlayingSceneIndex -= 1;
					scope.playExperience();
				} else {
					scope.decrementSceneOnPreviousSceneButtonClick();
					scope.calculateTotalScenesDuration();
					var playSceneInformation = scope.calculateInformationToPlayScene(scope.currentPlayingSceneIndex);
					var playedPercentage = scope.getPercentageFromSeconds(playSceneInformation.playedSeconds);
					scope.previewExperienceAtPercentage(playedPercentage);
				}
			});
		},
		decrementSceneOnPreviousSceneButtonClick: function () {
			if (this.currentPlayingSceneIndex <= -1) {
				this.currentPlayingSceneIndex = 0;
				return;
			}

			this.currentPlayingSceneIndex -= 1;
		},
		configureFullScreenButton: function (button) {
			var scope = this;
			scope.fullScreenButton = button;
			scope.fullScreenButton.addEventListener("click", function () {
				scope.handleFullScreenClick();
			});
		},
		configureExperiencePlayerControlsContainer: function (experiencePlayerControlsContainer) {
			this.experiencePlayerControlsContainer = experiencePlayerControlsContainer;
		},
		configureIframe: function (iframe) {
			if (!iframe) {
				this.iframeContainer = null;
				return;
			}

			if (iframe === this.iframeContainer) {
				return;
			}

			this.iframeContainer = iframe;
			this.iframeContainer.documentElement.setAttribute("style", "background-color:white; padding:8px");

			var playButton = iframe.querySelector('.experience4d-box');
			this.configurePlayButton(playButton);

			var seekBar = iframe.querySelector('.experience4d-seek-bar');
			this.configureSeekBar(seekBar);

			this.currentPlayingSceneNumberElement = iframe.querySelector('.experience4d-video-controls-scene-number');
			this.currentPlayingSceneTitleElement = iframe.querySelector('.experience4d-video-controls-scene-title');
			this.currentPlayedTimeElement = iframe.querySelector('.experience4d-time-count');

			var previousSceneButton = iframe.querySelector('.experience4d-previous-scene');
			this.configurePreviousSceneButton(previousSceneButton);

			var nextSceneButton = iframe.querySelector('.experience4d-next-scene');
			this.configureNextSceneButton(nextSceneButton);

			var fullScreenButton = iframe.querySelector('.experience4d-full-screen');
			this.configureFullScreenButton(fullScreenButton);

			var experiencePlayerControlsContainer = iframe.querySelector(".experience4d-video-controls-container");
			this.configureExperiencePlayerControlsContainer(experiencePlayerControlsContainer);

			this.modelViewer = iframe.querySelector("model-viewer");
		},
		configureModelViewer: function () {
			if (!this.modelViewer) {
				console.error("model-viewer is not defined for 4D experience player iframe");
				return;
			}
			if (this.modelViewer.loaded) {
				this.previewExperienceAtPercentage(0);
				this.experiencePlayerControlsContainer.style.opacity = 1;
				return;
			}

			var scope = this;
			this.modelViewer.addEventListener("load", function () {
				scope.previewExperienceAtPercentage(0);
				scope.experiencePlayerControlsContainer.style.opacity = 1;
			});
		},
		configurePlayer: function (iframe, animations, scenes, editor, playingScope, getAnimationsCallback,
								   getScenesCallback, playExperienceCallback, experienceConfig) {
			this.clearPlayerTimeoutsAndIntervals();

			//reset default values
			this.totalDuration = 0;
			this.startPlayingMilliseconds = null;
			this.shouldResumePlaying = false;
			this.shouldResumePlayingFromBeginningOfScene = false;
			this.interpolationDecay = 50;

			this.configureIframe(iframe);
			if (!this.iframeContainer) {
				console.error("iframe not defined for 4D experience player");
				return;
			}

			this.playingAnimations = animations;
			this.playingScenes = scenes;
			this.currentPlayingAnimationIndex = -1;
			this.currentPlayingSceneIndex = -1;
			this.playingScope = playingScope;
			this.editor = editor;
			this.getAnimationsCallback = getAnimationsCallback;
			this.getScenesCallback = getScenesCallback;
			this.playExperienceCallback = playExperienceCallback;

			this.configureModelViewer();
			this.applyExperienceConfig(experienceConfig);
		},
		applyExperienceConfig: function (experienceConfig) {
			if (!experienceConfig) {
				return;
			}

			this.configureInterpolationDecay(experienceConfig.interpolationDecay);
		},
		configureInterpolationDecay: function (interpolationDecay) {
			if (interpolationDecay) {
				var interpolationDecayValue = parseFloat(interpolationDecay);
				if (isNaN(interpolationDecayValue) || interpolationDecayValue < 0) {
					return;
				}
				this.interpolationDecay = interpolationDecayValue;
			}
		},
		configureHtmlSceneElements: function (sceneHtmlElements) {
			if (!sceneHtmlElements) {
				this.sceneHtmlElements = [];
				return;
			}

			this.sceneHtmlElements = sceneHtmlElements;
		},
		configureAnimations: function () {
			if (!this.getAnimationsCallback) {
				return;
			}
			this.playingAnimations = this.getAnimationsCallback();
		},
		configureScenes: function () {
			if (!this.getScenesCallback) {
				return;
			}
			this.playingScenes = this.getScenesCallback();
		},
		handleFullScreenClick: function () {
			if (
				document.fullscreenElement ||
				document.webkitFullscreenElement ||
				document.mozFullScreenElement ||
				document.msFullscreenElement
			) {
				if (document.exitFullscreen) {
					document.exitFullscreen();
				} else if (document.mozCancelFullScreen) {
					document.mozCancelFullScreen();
				} else if (document.webkitExitFullscreen) {
					document.webkitExitFullscreen();
				} else if (document.msExitFullscreen) {
					document.msExitFullscreen();
				}
			} else {
				var element = this.iframeContainer.documentElement;
				if (element.requestFullscreen) {
					element.requestFullscreen();
				} else if (element.mozRequestFullScreen) {
					element.mozRequestFullScreen();
				} else if (element.webkitRequestFullscreen) {
					element.webkitRequestFullscreen(Element.ALLOW_KEYBOARD_INPUT);
				} else if (element.msRequestFullscreen) {
					element.msRequestFullscreen();
				}
			}
		}
	};
};