var Experience4DUtil = (function () {
	return {
		getAnimationsFromExperienceDoc: function (experienceDoc) {
			var animationElements = experienceDoc.querySelectorAll("Animation");
			return Array.from(animationElements).map(function (elem) {
				var name = elem.getAttribute('id');
				var startTime = Number(elem.getAttribute('start')) || 0;
				var stopTime = Number(elem.getAttribute('stop')) || null;
				return {
					name: name,
					stopTime: stopTime,
					startTime: startTime
				};
			});
		},
		getScenesFromExperienceDoc: function (experienceDoc) {
			var sceneElements = experienceDoc.querySelectorAll("Scene");
			return Array.from(sceneElements).map(function (element) {
				var duration = element.getAttribute('duration');
				duration = isNaN(duration) ? 0 : Number(duration);
				var stopAtEnd = element.getAttribute('stopatend') === 'true';
				var title = element.querySelector("Title").textContent || "";
				var cameraPositionElem = element.querySelector("CameraPosition");
				var cameraOrbit = cameraPositionElem.getAttribute("cameraOrbit");
				var cameraTarget = cameraPositionElem.getAttribute("cameraTarget");
				var fieldOfView = cameraPositionElem.getAttribute("fieldOfView");

				return {
					duration: duration,
					stopAtEnd: stopAtEnd,
					title: title,
					cameraOrbit: cameraOrbit,
					cameraTarget: cameraTarget,
					fieldOfView: fieldOfView
				};
			});
		},
		getModelOriginalUriFromExperienceDoc: function (experienceDoc) {
			var modelRefControl = experienceDoc.querySelector("RefControl");
			if (!modelRefControl) {
				return null;
			}

			var originalUri = modelRefControl.getAttribute("originalURL");
			if (!originalUri) {
				return null;
			}

			var serverUri = modelRefControl.getAttribute("serverURL");
			if (!serverUri) {
				return null;
			}

			return serverUri + originalUri;
		},
		getExperienceConfigFromExperienceDoc: function(experienceDoc) {
			var experienceConfig = {}
			var experienceConfigElem = experienceDoc.querySelector("ExperienceConfig");
			if (!experienceConfigElem) {
				return experienceConfig;
			}

			var interpolationDecay = experienceConfigElem.getAttribute("interpolationDecay");
			if (interpolationDecay) {
				experienceConfig.interpolationDecay = interpolationDecay;
			}

			return experienceConfig;
		},
		displaySceneContentInHTML5: function (scope, modelViewer, currentPlayingSceneIndex) {
			if (scope.sceneHtmlElements.length === 0) {
				return;
			}

			var iframeDoc = modelViewer.ownerDocument;
			var panel = iframeDoc.querySelector('.panel-container');
			var modelViewerContainer = iframeDoc.querySelector('.model-viewer-container');
			if (currentPlayingSceneIndex >= scope.sceneHtmlElements.length) {
				Experience4DUtil.hideSceneContentPanel(panel, modelViewerContainer);
				return;
			}

			Experience4DUtil.showSceneContentPanel(panel, modelViewerContainer);
			var sceneContentHtml = iframeDoc.querySelector("[class*='scene-panel-body']");
			// clean scene content
			sceneContentHtml.innerHTML = "";
			if (currentPlayingSceneIndex < scope.sceneHtmlElements.length) {
				var scenePage = scope.sceneHtmlElements[currentPlayingSceneIndex].cloneNode(true);
				scenePage.removeAttribute("style");
				sceneContentHtml.append(scenePage);
			}
		},
		showSceneContentPanel: function (panel, modelViewerContainer) {
			if (modelViewerContainer.classList.contains("col-xs-12")) {
				modelViewerContainer.classList.remove("col-xs-12");
			}

			if (!modelViewerContainer.classList.contains("col-xs-6")) {
				modelViewerContainer.classList.add("col-xs-6");
			}

			if (panel.classList.contains('hidden')) {
				panel.classList.remove('hidden');
			}
			if (!panel.classList.contains('visible')) {
				panel.classList.add('visible');
			}
		},
		hideSceneContentPanel: function (panel, modelViewerContainer) {
			if (modelViewerContainer.classList.contains("col-xs-6")) {
				modelViewerContainer.classList.remove("col-xs-6");
			}

			if (!modelViewerContainer.classList.contains("col-xs-12")) {
				modelViewerContainer.classList.add("col-xs-12");
			}

			if (panel.classList.contains('visible')) {
				panel.classList.remove('visible');
			}
			if (!panel.classList.contains('hidden')) {
				panel.classList.add('hidden');
			}
		},
		handleHTML5ExperienceLoad: function (event, scope) {
			var iframe = scope.frameElement;
			var modelViewer = event.target.ownerDocument.getElementsByTagName("model-viewer")[0];
			if (!modelViewer) {
				return;
			}
			var currentNode = iframe.getAttribute("currentNode");
			var parser = new DOMParser();
			var experienceDocument = parser.parseFromString(currentNode, "application/xml");
			var animations = this.getAnimationsFromExperienceDoc(experienceDocument);
			var scenes = this.getScenesFromExperienceDoc(experienceDocument);
			var sceneHtmlElements = iframe.parentElement.querySelectorAll("[class='scene-page']");
			if (!scope.experience4DPlayer) {
				scope.experience4DPlayer = new Experience4DPlayer();
			}

			var experienceConfig = this.getExperienceConfigFromExperienceDoc(experienceDocument);
			scope.experience4DPlayer.configureHtmlSceneElements(sceneHtmlElements);
			scope.experience4DPlayer.configurePlayer(iframe.contentDocument, animations, scenes, null, scope,
				null, null, null, experienceConfig);
		},
		handleHTML5ModelError: function (event, iconPath, smcScope) {
			var iframeDoc = event.target.ownerDocument;
			var bodyElement = iframeDoc.body;

			var errorMessage;
			if (smcScope) {
				var translatorCtrl = smcScope.System.getTranslatorCtrl();
				errorMessage = translatorCtrl.t("smc.model3d.error.webserver");
			} else {
				errorMessage = "Preview of this 3D model requires a webserver";
			}

			bodyElement.innerHTML = "";

			// create an image element
			var img = iframeDoc.createElement('img');
			img.src = iconPath;
			img.alt = errorMessage;
			img.style.width = '60px';
			img.style.height = '60px';

			// create a p element with custom error message
			var p = iframeDoc.createElement('p');
			p.textContent = errorMessage;
			p.title = errorMessage;
			p.style.fontSize = '12px';
			p.style.fontFamily = 'Tahoma, Helvetica, sans-serif';


			// insert the image and paragraph elements
			bodyElement.appendChild(img);
			bodyElement.appendChild(p);
		},
		onLoadExperience4DIframe: function (event) {
			var iframeSrcDoc = event.target.textContent;
			var doc = event.target.contentDocument;
			doc.open();
			doc.write(iframeSrcDoc);
			doc.close();
		}
	};
})();